(* file: change.ml
 * author: Bob Muller
 *
 * CS1101 Computer Science I
 *
 * This code implements a greedy algorithm for making change.
 *)
let rec maxList xs =
  match xs with
  | [] -> failwith "Bad input"
  | [x] -> x
  | x :: xs -> max x (maxList xs)

let rec remove coin coins =
  match coins with
  | [] -> []
  | coin' :: coins -> (match coin = coin' with
                       | true  -> coins
                       | false -> coin' :: (remove coin coins))

let rec copy item n =
  match n = 0 with
  | true  -> []
  | false -> item :: (copy item (n - 1))

(* getLargest : int list -> int * int list
 *)                       
let getLargest coins =
  let maxCoin = maxList coins
  in
  (maxCoin, remove maxCoin coins)
    
(* change : int -> int list -> int list
 *
 * The call (change amount coins) returns the smallest list of
 * coins from coins that add up to amount. This is a greedy algorithm,
 * it doesn't work for all sets of coins but it does work for US
 * coinage.
 *)
let rec change amount coins =
  match amount = 0 with
  | true  -> []
  | false -> let (largest, rest) = getLargest coins in
             let numLargest = amount / largest in
             let residualAmount = amount mod largest in
             let largestCoins = copy largest numLargest
             in
             largestCoins @ (change residualAmount rest)
